% a m script to do stereo localization using a binaural impulse response
% we assume a listener with an interaural distance, facing the front speakers
% to start, the listener is centered, and the amplitude ratio varies.
% we determine the net interaural time difference

sr = 44100;
ss = 1100;  % speed of sound in feet per second

plot_frequency = 300;
ear_spacing = 21;
shadow_freq = 2000;	% -3dB frequency for head shadowing
pz = 1/20;		%pole/zero ratio for head shadowing

ar = 2*pi/360;
sh_frq = 1-exp(-2*pi*shadow_freq/sr)
shadow_delay = (1-pz)/(8*shadow_freq); %filter group delay in seconds

% speaker positions sxl = x position left speaker, sxr = x position right speaker, sy = y position
% listener positions rx_av, ry.

sxl = -3;
sxr = 3;


sy = 0;

rx_av = 0;
ry = 3;

fnumb = 1500;  % goes to 1.2kHz

speaker_angle = atan(sxr/ry)/ar

delta = .5*ear_spacing/2.54;	%interaural spacing in inches   
delta = delta/12;	%interaural spacing in feet

% int_time is the total time between the ears for a signal arriving at 90 degrees
int_time = 2*delta/ss;

anumb = 45;

deltime = zeros(1,anumb);
delfrq = 22050/(8192*2);  %11025/8192;
% distance to sample number:  d*sr/ss
dsn = sr/ss; % convert time in seconds to sample number


plot_freq = plot_frequency/delfrq;


for a_index = 1:anumb+1

	a_angle = ar*45*(a_index-1)/anumb;

	il = zeros(size(1:8192*4));  % plot ~2/3 second at 44100
	ir = zeros(size(1:8192*4));  % plot ~2/3 second at 44100
	ilr = zeros(size(1:8192*4));  % plot ~2/3 second at 44100
	irl = zeros(size(1:8192*4));  % plot ~2/3 second at 44100

	lamp = sin(a_angle);
	ramp = cos(a_angle);

	% find impulse response from left speaker to left ear
	rx = rx_av-delta;
	dll2 = (rx-sxl)^2+(ry-sy)^2;
	dll = sqrt(dll2);
	all = lamp/dll2;

	% from right speaker to left ear
	drl2 = (rx-sxr)^2+(ry-sy)^2;
	drl = sqrt(drl2);
	arl = ramp/drl2;
	% from left speaker to right ear
	rx = rx_av+delta;
	dlr2 = (rx-sxl)^2+(ry-sy)^2;
	dlr = sqrt(dlr2);
	alr = lamp/dlr2;
	% from right speaker to right ear
	drr2 = (rx-sxr)^2+(ry-sy)^2;
	drr = sqrt(drr2);
	arr = ramp/drr2;

%	compensate for head shadow filter group delay
%	drl is currently in feet, so we must convertg shadow_delay to feet
	drl = drl - shadow_delay*ss;
	dlr = dlr - shadow_delay*ss;

	% clear space for the impulse response
	% find the longest delay
	ir_ln = dlr*dsn+100;
	if drl > dlr
		ir_ln = drl*dsn+100;
	end

	% combine the impuse responses

	dll = dll*dsn;
	rem = dll-floor(dll);
	il(floor(dll)) = il(floor(dll))+(1-rem)*all;
	il(floor(dll)+1) = il(floor(dll+1))+(rem)*all;

	drl = drl*dsn;
	rem = drl-floor(drl);
	irl(floor(drl)) = irl(floor(drl))+(1-rem)*arl;
	irl(floor(drl)+1) = irl(floor(drl+1))+(rem)*arl;

	dlr = dlr*dsn;
	rem = dlr-floor(dlr);
	ilr(floor(dlr)) = ir(floor(dlr))+(1-rem)*alr;
	ilr(floor(dlr)+1) = ir(floor(dlr+1))+(rem)*alr;

	drr = drr*dsn;
	rem = drr-floor(drr);
	ir(floor(drr)) = ir(floor(drr))+(1-rem)*arr;
	ir(floor(drr)+1) = ir(floor(drr+1))+(rem)*arr;

	% do the shadow filter
	ilrf = 0;
	irlf = 0;
	for i = 1:ir_ln
		ilrf = (sh_frq*(1-pz))*ilr(i)+ilrf-sh_frq*ilrf;
		irlf = (sh_frq*(1-pz))*irl(i)+irlf-sh_frq*irlf;
		ilro(i) = pz*ilr(i)+ilrf;
		irlo(i) = pz*irl(i)+irlf;
	end

	% combine direct and cross terms
	il(1:ir_ln) = il(1:ir_ln)+irlo;
	ir(1:ir_ln) = ir(1:ir_ln)+ilro;

%	plot(il(1:200))
%	hold on
%	plot(ir(1:200))
%	hold off

	fftr = fft(ir);
	fftl = fft(il);

	aftl = angle(fftl);
	aftr = angle(fftr);

	% find the phase difference with a C program

	phase_diff = phasedif(aftl,aftr,fnumb);

	for i=1:fnumb
		if phase_diff(i) >= pi;
			phase_diff(i) = phase_diff(i)-2*pi;
		end
	end

	% convert to ITD - delphase = 2*pi*deltime*f;
	% so deltime = delphase/(2*pi*f)
	%if a_index == 44
	%stop
	%end
	n = 1:fnumb;
	% this is the delay difference in milliseconds
	deltime(a_index,1:fnumb) = 1000*phase_diff./(2*pi*delfrq*(n-1));

	%plot(deltime(a_index,1:fnumb))
	%pause
end

for j = 1:6
% convert the time in ms to an angle
angs = asin(deltime(1:anumb+1,plot_freq)/(1000*int_time))/ar;

plot(0:anumb,angs,'w');
xlabel('pan pot position in degrees')
ylabel('apparent image position - left at bottom')

hold on

plot_frequency = plot_frequency + 150
plot_freq = plot_frequency/delfrq;
end

ar = 2*pi/360;
ra = 1/ar;

th = ar*(1:45);

g1 = sin(th);
g2 = cos(th);

th2 = ra*asin(sin(ar*45)*(g1-g2)./(g1+g2));

th3 = ra*atan(tan(ar*45)*(g1-g2)./(g1+g2));

plot(ra*th,th2,'w:')
plot(ra*th,th3,'w--')
hold off
