% stereo1 - where the right speaker is in the center
% a m script to do stereo localization using a binaural impulse response
% we assume a listener with an interaural distance, facing the front speakers
% to start, the listener is centered, and the amplitude ratio varies.
% we determine the net interaural time difference

sr = 44100;
ss = 1100;  % speed of sound in feet per second

ar = 2*pi/360;

% speaker positions sxl = x position left speaker, sxr = x position right speaker, sy = y position
% listener positions rx_av, ry.

sxl = -3;
sxr = 0;


sy = 0;

rx_av = 0;
ry = 3;

fnumb = 900;  % goes to 1.2kHz

speaker_angle = atan(sxr/ry)/ar

delta = .5*25/2.54;	%interaural spacing in inches   
delta = delta/12;	%interaural spacing in feet

int_time = 2*delta/ss;

anumb = 45;

deltime = zeros(fnumb,anumb);



for a_index = 1:anumb+1

	a_angle = ar*45*(a_index-1)/anumb;

	il = zeros(size(1:8192*4));  % plot ~2/3 second at 44100
	ir = zeros(size(1:8192*4));  % plot ~2/3 second at 44100

	lamp = sin(a_angle);
	ramp = cos(a_angle);

	% find impulse response from left speaker to left ear
	rx = rx_av-delta;
	dll2 = (rx-sxl)^2+(ry-sy)^2;
	dll = sqrt(dll2);
	all = lamp/dll2;

	% from right speaker to left ear
	drl2 = (rx-sxr)^2+(ry-sy)^2;
	drl = sqrt(drl2);
	arl = ramp/drl2;
	% from left speaker to right ear
	rx = rx_av+delta;
	dlr2 = (rx-sxl)^2+(ry-sy)^2;
	dlr = sqrt(dlr2);
	alr = lamp/dlr2;
	% from right speaker to right ear
	drr2 = (rx-sxr)^2+(ry-sy)^2;
	drr = sqrt(drr2);
	arr = ramp/drr2;

	% distance to sample number:  d*sr/ss
	dsn = sr/ss;


	% combine the impuse responses

	dll = dll*dsn;
	rem = dll-floor(dll);
	il(floor(dll)) = il(floor(dll))+(1-rem)*all;
	il(floor(dll)+1) = il(floor(dll+1))+(rem)*all;
	drl = drl*dsn;
	rem = drl-floor(drl);
	il(floor(drl)) = il(floor(drl))+(1-rem)*arl;
	il(floor(drl)+1) = il(floor(drl+1))+(rem)*arl;

	dlr = dlr*dsn;
	rem = dlr-floor(dlr);
	ir(floor(dlr)) = ir(floor(dlr))+(1-rem)*alr;
	ir(floor(dlr)+1) = ir(floor(dlr+1))+(rem)*alr;
	drr = drr*dsn;
	rem = drr-floor(drr);
	ir(floor(drr)) = ir(floor(drr))+(1-rem)*arr;
	ir(floor(drr)+1) = ir(floor(drr+1))+(rem)*arr;

	fftr = fft(ir);
	fftl = fft(il);

	aftl = angle(fftl);
	aftr = angle(fftr);

	% find the phase difference with a C program

	phase_diff = phasedif(aftl,aftr,fnumb);

	% convert to ITD - delphase = 2*pi*deltime*f;
	% so deltime = delphase/(2*pi*f)

	n = 1:fnumb;
	delfrq = 22050/(8192*2);  %11025/8192;

	deltime(a_index,1:fnumb) = 1000*phase_diff./(2*pi*delfrq*(n-1));

	%plot(deltime(a_index,1:fnumb))
	%pause
end

angs = asin(deltime(1:anumb+1,600)/(1000*int_time))/ar;

plot(0:anumb,angs);
hold on
angs = asin(deltime(1:anumb+1,100)/(1000*int_time))/ar;

plot(0:anumb,angs,'r');
angs = asin(deltime(1:anumb+1,300)/(1000*int_time))/ar;

plot(0:anumb,angs,'b');



hold off
xlabel('pan pot position in degrees')
ylabel('apparent image position - left at bottom')

ar = 2*pi/360;
ra = 1/ar;

th = ar*(1:45);

g1 = sin(th);
g2 = cos(th);

th2 = ra*asin(sin(ar*45)*(g1-g2)./(g1+g2));

th3 = ra*atan(tan(ar*45)*(g1-g2)./(g1+g2));

%hold on

%plot(ra*th,th2,'b')
%plot(ra*th,th3,'r')
%hold off
